----------------------------------------------------------------------------------
-- Company: Digilent RO
-- Engineer: Mircea Dabacan
-- 
-- Create Date:    17:38:30 04/20/2011 
-- Module Name:    segDisplay - Behavioral 
-- Project Name:   Hb5RefProj
-- Description: This module decodes and outputs on the SSG Display the direction and the
--               Speed (rot/min) or the Duty Factor selected using the switches on-board. 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity segDisplay is
    Port ( ck : in  STD_LOGIC;
           vecDfIn : in  STD_LOGIC_VECTOR (7 downto 0);
           swDir : in  STD_LOGIC;
           SpeedIn : in  STD_LOGIC_VECTOR (12 downto 0);
           swSelDisp : in  STD_LOGIC;  -- selects the 7 seg display: '0' = Duty Factor, '1' = Speed
           an : out  STD_LOGIC_VECTOR (3 downto 0);
           seg : out  STD_LOGIC_VECTOR (6 downto 0));
end segDisplay;

architecture Behavioral of segDisplay is
------------------------------------------------------------------------
-- Constant and Signal Declarations
------------------------------------------------------------------------
  signal seg1, seg2: std_logic_vector(6 downto 0);  -- segments for DF, position, etc.
  signal HEX1, HEX2 : std_logic_vector(3 downto 0);  -- a digit for 7 seg display for DF, Position, etc.
  signal cnt : std_logic_vector(18 downto 0):="0000000000000000000";-- divider for 7SD
  signal sign: std_logic_vector(3 downto 0);
------------------------------------------------------------------------
-- Module Implementation
------------------------------------------------------------------------

begin
-- Direction of the Speed measured using the Hall transducers
sign <= "1110" when (SpeedIn(12) = '0') else "1111";

  -- displaying the BCD value 
div: process (ck)
 begin
  if rising_edge(ck) then
   cnt <=cnt + "0000000000000000001";
  end if;
 end process;

with cnt(18 downto 17) select
 hex1 <= vecDfIn(3 downto 0) when "00",  -- for Duty Factor
        vecDfIn(7 downto 4) when "01",
        "111" & swDir     when "10",  -- "1110" for positive, "1111" for negative
        "1110" when others; -- MSD off

with cnt(18 downto 17) select
 hex2 <= SpeedIn(3 downto 0) when "00",  -- for speed
        SpeedIn(7 downto 4) when "01",
        SpeedIn(11 downto 8) when "10",
        sign when others; -- positive or negative 

with cnt(18 downto 17) select
 an <= "1110" when "00",
       "1101" when "01",
       "1011" when "10",
       "0111" when others; 

--HEX-to-seven-segment decoder
-- 
-- segment encoding
--      0
--     ---  
--  5 |   | 1
--     ---   <- 6
--  4 |   | 2
--     ---
--      3

    with HEX1 select
   seg1<= "1000000" when "0000",   --0
         "1111001" when "0001",   --1
         "0100100" when "0010",   --2
         "0110000" when "0011",   --3
         "0011001" when "0100",   --4
         "0010010" when "0101",   --5
         "0000010" when "0110",   --6
         "1111000" when "0111",   --7
         "0000000" when "1000",   --8
         "0010000" when "1001",   --9
         "0111111" when "1111",   -- minus sign
         "1111111" when others;   -- nothing, for plus sign
    with HEX2 select
   seg2<= "1000000" when "0000",   --0
         "1111001" when "0001",   --1
         "0100100" when "0010",   --2
         "0110000" when "0011",   --3
         "0011001" when "0100",   --4
         "0010010" when "0101",   --5
         "0000010" when "0110",   --6
         "1111000" when "0111",   --7
         "0000000" when "1000",   --8
         "0010000" when "1001",   --9
         "0111111" when "1111",   -- minus sign
         "1111111" when others;   -- nothing, for plus sign
  seg <= seg1 when swSelDisp = '0' else seg2;		
end Behavioral;

